@@ -121,34 +121,14 @@ def load_settings(session_path):
     return settings
 
 
-def _load_encoder_events_file_lt5(file_path):
-    """
-    File loader without the session overhead
-    :param file_path:
-    :return: dataframe of encoder events
-    """
-    if file_path.stat().st_size == 0:
-        logger_.error("_iblrig_encoderEvents.raw.ssv is an empty file. ")
-        raise ValueError("_iblrig_encoderEvents.raw.ssv is an empty file. ABORT EXTRACTION. ")
-    data = pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False)
-    data = data.drop([0, 2, 5], axis=1)
-    data.columns = ['re_ts', 'sm_ev', 'bns_ts']
-    return _groom_wheel_data_lt5(data, label='_iblrig_encoderEvents.raw.ssv', path=file_path)
+def load_stim_position_screen(session_path):
+    path = Path(session_path).joinpath("raw_behavior_data")
+    path = next(path.glob("_iblrig_stimPositionScreen.raw*.csv"), None)
 
-
-def _load_encoder_events_file_ge5(file_path):
-    """
-    File loader without the session overhead
-    :param file_path:
-    :return: dataframe of encoder events
-    """
-    if file_path.stat().st_size == 0:
-        logger_.error("_iblrig_encoderEvents.raw.ssv is an empty file. ")
-        raise ValueError("_iblrig_encoderEvents.raw.ssv is an empty file. ABORT EXTRACTION. ")
-    data = pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False)
-    data = data.drop([2], axis=1)
-    data.columns = ['re_ts', 'sm_ev']
-    return _groom_wheel_data_ge5(data, label='_iblrig_encoderEvents.raw.ssv', path=file_path)
+    data = pd.read_csv(path, sep=',', header=None, error_bad_lines=False)
+    data.columns = ['contrast', 'position', 'bns_ts']
+    data[2] = pd.to_datetime(data[2])
+    return data
 
 
 def load_encoder_events(session_path, settings=False):
@@ -157,17 +137,11 @@ def load_encoder_events(session_path, settings=False):
 
     Assumes that a folder called "raw_behavior_data" exists in folder.
 
-    On each trial the RE sends 3 events to Bonsai 1 - meaning trial start/turn
-    off the stim; 2 - meaning show the current trial stimulus; and 3 - meaning
-    begin the closed loop making the stim move with the RE. These events are
-    triggered by the state machine in the corrensponding states: trial_start,
-    stim_on, closed_loop
-
-    Raw datafile Columns:
-        Event, RE timestamp, Source, data, Bonsai Timestamp
-
-    Event is always equal 'Event' Source is always equal 'StateMachine'. For
-    this reason these columns are dropped.
+    Events number correspond to following bpod states:
+    1: correct / hide_stim
+    2: stim_on
+    3: closed_loop
+    4: freeze_error / freeze_correct
 
     >>> data.columns
     >>> ['re_ts',   # Rotary Encoder Timestamp (ms) 'numpy.int64'
@@ -188,7 +162,11 @@ def load_encoder_events(session_path, settings=False):
         settings = load_settings(session_path)
     if settings is None or settings['IBLRIG_VERSION_TAG'] == '':
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
-
+        # auto-detect old files when version is not labeled
+        with open(path) as fid:
+            line = fid.readline()
+        if line.startswith('Event') and 'StateMachine' in line:
+            settings = {'IBLRIG_VERSION_TAG': '0.0.0'}
     if not path:
         return None
     if version.ge(settings['IBLRIG_VERSION_TAG'], '5.0.0'):
@@ -197,28 +175,62 @@ def load_encoder_events(session_path, settings=False):
         return _load_encoder_events_file_lt5(path)
 
 
-def _load_encoder_positions_file_lt5(file_path):
-    # file loader without the session overhead
+def _load_encoder_ssv_file(file_path, **kwargs):
+    file_path = Path(file_path)
     if file_path.stat().st_size == 0:
-        logger_.error("_iblrig_encoderPositions.raw.ssv is an empty file. ")
-        raise ValueError("_iblrig_encoderPositions.raw.ssv is an empty file. ABORT EXTRACTION. ")
-    data = pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False)
-    data = data.drop([0, 4], axis=1)
-    data.columns = ['re_ts', 're_pos', 'bns_ts']
+        logger_.error(f"{file_path.name} is an empty file. ")
+        raise ValueError(f"{file_path.name} is an empty file. ABORT EXTRACTION. ")
+    return pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False, **kwargs)
+
+
+def _load_encoder_positions_file_lt5(file_path):
+    """
+    File loader without the session overhead
+    :param file_path:
+    :return: dataframe of encoder events
+    """
+    data = _load_encoder_ssv_file(file_path,
+                                  names=['_', 're_ts', 're_pos', 'bns_ts', '__'],
+                                  usecols=['re_ts', 're_pos', 'bns_ts'])
     return _groom_wheel_data_lt5(data, label='_iblrig_encoderPositions.raw.ssv', path=file_path)
 
 
 def _load_encoder_positions_file_ge5(file_path):
-    # file loader without the session overhead
-    if file_path.stat().st_size == 0:
-        logger_.error("_iblrig_encoderPositions.raw.ssv is an empty file. ")
-        raise ValueError("_iblrig_encoderPositions.raw.ssv is an empty file. ABORT EXTRACTION. ")
-    data = pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False)
-    data = data.drop([2], axis=1)
-    data.columns = ['re_ts', 're_pos']
+    """
+    File loader without the session overhead
+    :param file_path:
+    :return: dataframe of encoder events
+    """
+    data = _load_encoder_ssv_file(file_path,
+                                  names=['re_ts', 're_pos', '_'],
+                                  usecols=['re_ts', 're_pos'])
     return _groom_wheel_data_ge5(data, label='_iblrig_encoderPositions.raw.ssv', path=file_path)
 
 
+def _load_encoder_events_file_lt5(file_path):
+    """
+    File loader without the session overhead
+    :param file_path:
+    :return: dataframe of encoder events
+    """
+    data = _load_encoder_ssv_file(file_path,
+                                  names=['_', 're_ts', '__', 'sm_ev', 'bns_ts', '___'],
+                                  usecols=['re_ts', 'sm_ev', 'bns_ts'])
+    return _groom_wheel_data_lt5(data, label='_iblrig_encoderEvents.raw.ssv', path=file_path)
+
+
+def _load_encoder_events_file_ge5(file_path):
+    """
+    File loader without the session overhead
+    :param file_path:
+    :return: dataframe of encoder events
+    """
+    data = _load_encoder_ssv_file(file_path,
+                                  names=['re_ts', 'sm_ev', '_'],
+                                  usecols=['re_ts', 'sm_ev'])
+    return _groom_wheel_data_ge5(data, label='_iblrig_encoderEvents.raw.ssv', path=file_path)
+
+
 def load_encoder_positions(session_path, settings=False):
     """
     Load Rotary Encoder (RE) positions from raw data file within a session path.
@@ -241,7 +253,7 @@ def load_encoder_positions(session_path, settings=False):
          'bns_ts']  # Bonsai Timestamp                  'pandas.Timestamp'
         # pd.to_datetime(data.bns_ts) to work in datetimes
 
-    :param session_path: Absoulte path of session folder
+    :param session_path: Absolute path of session folder
     :type session_path: str
     :return: dataframe w/ 3 cols and N positions
     :rtype: Pandas.DataFrame
@@ -254,7 +266,11 @@ def load_encoder_positions(session_path, settings=False):
         settings = load_settings(session_path)
     if settings is None or settings['IBLRIG_VERSION_TAG'] == '':
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
-
+        # auto-detect old files when version is not labeled
+        with open(path) as fid:
+            line = fid.readline()
+        if line.startswith('Position'):
+            settings = {'IBLRIG_VERSION_TAG': '0.0.0'}
     if not path:
         return None
     if version.ge(settings['IBLRIG_VERSION_TAG'], '5.0.0'):
@@ -283,12 +299,13 @@ def load_encoder_trial_info(session_path):
          'stim_angle',    # Angle of Gabor 0 = Vertical      'numpy.float64'
          'stim_gain',     # Wheel gain (mm/º of stim)        'numpy.float64'
          'stim_sigma',    # Size of patch                    'numpy.float64'
+         'stim_phase',    # Phase of gabor                    'numpy.float64'
          'bns_ts' ]       # Bonsai Timestamp                 'pandas.Timestamp'
         # pd.to_datetime(data.bns_ts) to work in datetimes
 
     :param session_path: Absoulte path of session folder
     :type session_path: str
-    :return: dataframe w/ 8 cols and ntrials lines
+    :return: dataframe w/ 9 cols and ntrials lines
     :rtype: Pandas.DataFrame
     """
     if session_path is None:
@@ -298,10 +315,11 @@ def load_encoder_trial_info(session_path):
     if not path:
         return None
     data = pd.read_csv(path, sep=' ', header=None)
-    data = data.drop([8], axis=1)
+    data = data.drop([9], axis=1)
     data.columns = ['trial_num', 'stim_pos_init', 'stim_contrast', 'stim_freq',
-                    'stim_angle', 'stim_gain', 'stim_sigma', 'bns_ts']
-    return _groom_wheel_data_lt5(data, label='_iblrig_encoderEvents.raw.ssv', path=path)
+                    'stim_angle', 'stim_gain', 'stim_sigma', 'stim_phase', 'bns_ts']
+    # return _groom_wheel_data_lt5(data, label='_iblrig_encoderEvents.raw.ssv', path=path)
+    return data
 
 
 def load_ambient_sensor(session_path):
@@ -357,21 +375,20 @@ def load_mic(session_path):
     return data
 
 
-def _groom_wheel_data_lt5(data, label='file ', path=''):
-    """
-    The whole purpose of this function is to account for variability and corruption in
-    the wheel position files. There are many possible errors described below, but
-    nothing excludes getting new ones.
-    """
-    # sometimes the text file is cropped
+def _clean_wheel_dataframe(data, label, path):
     if np.any(data.isna()):
         logger_.warning(label + ' has missing/incomplete records \n %s', path)
+    # first step is to re-interpret as numeric objects if not already done
+    for col in data.columns:
+        if data[col].dtype == np.object and col not in ['bns_ts']:
+            data[col] = pd.to_numeric(data[col], errors='coerce')
+    # then drop Nans and duplicates
     data.dropna(inplace=True)
-    data.drop(data.loc[data.bns_ts.apply(len) != 33].index, inplace=True)
     data.drop_duplicates(keep='first', inplace=True)
     data.reset_index(inplace=True)
     # handle the clock resets when microseconds exceed uint32 max value
     drop_first = False
+    data['re_ts'] = data['re_ts'].astype(np.double, copy=False)
     if any(np.diff(data['re_ts']) < 0):
         ind = np.where(np.diff(data['re_ts']) < 0)[0]
         for i in ind:
@@ -395,10 +412,20 @@ def _groom_wheel_data_lt5(data, label='file ', path=''):
                 logger_.error(label + ' Rotary encoder timestamps are not sorted.' + str(path))
                 data.sort_values('re_ts', inplace=True)
                 data.reset_index(inplace=True)
-
     if drop_first is not False:
         data.drop(data.loc[:drop_first].index, inplace=True)
         data = data.reindex()
+    return data
+
+
+def _groom_wheel_data_lt5(data, label='file ', path=''):
+    """
+    The whole purpose of this function is to account for variability and corruption in
+    the wheel position files. There are many possible errors described below, but
+    nothing excludes getting new ones.
+    """
+    data = _clean_wheel_dataframe(data, label, path)
+    data.drop(data.loc[data.bns_ts.apply(len) != 33].index, inplace=True)
     # check if the time scale is in ms
     sess_len_sec = (datetime.strptime(data['bns_ts'].iloc[-1][:25], '%Y-%m-%dT%H:%M:%S.%f') -
                     datetime.strptime(data['bns_ts'].iloc[0][:25], '%Y-%m-%dT%H:%M:%S.%f')).seconds
@@ -415,43 +442,9 @@ def _groom_wheel_data_ge5(data, label='file ', path=''):
     the wheel position files. There are many possible errors described below, but
     nothing excludes getting new ones.
     """
-    # sometimes the text file is cropped
-    if np.any(data.isna()):
-        logger_.warning(label + ' has missing/incomplete records \n %s', path)
-    data.dropna(inplace=True)
-    data.drop_duplicates(keep='first', inplace=True)
-    data.reset_index(inplace=True)
-    # handle the clock resets when microseconds exceed uint32 max value
-    drop_first = False
-    if any(np.diff(data['re_ts']) < 0):
-        ind = np.where(np.diff(data['re_ts']) < 0)[0]
-        for i in ind:
-            # the first sample may be corrupt, in this case throw away
-            if i <= 1:
-                drop_first = i
-                logger_.warning(label + ' rotary encoder positions timestamps'
-                                        ' first sample corrupt ' + str(path))
-            # if it's an uint32 wraparound, the diff should be close to 2 ** 32
-            elif 32 - np.log2(data['re_ts'][i] - data['re_ts'][i + 1]) < 0.2:
-                data.loc[i + 1:, 're_ts'] = data.loc[i + 1:, 're_ts'] + 2 ** 32
-            # there is also the case where 2 positions are swapped and need to be swapped back
-
-            elif data['re_ts'][i] > data['re_ts'][i + 1] > data['re_ts'][i - 1]:
-                logger_.warning(label + ' rotary encoder timestamps swapped at index: ' +
-                                str(i) + '  ' + str(path))
-                a, b = data.iloc[i].copy(), data.iloc[i + 1].copy()
-                data.iloc[i], data.iloc[i + 1] = b, a
-            # if none of those 3 cases apply, raise an error
-            else:
-                logger_.error(label + ' Rotary encoder timestamps are not sorted.' + str(path))
-                data.sort_values('re_ts', inplace=True)
-                data.reset_index(inplace=True)
-
-    if drop_first is not False:
-        data.drop(data.loc[:drop_first].index, inplace=True)
-        data = data.reindex()
+    data = _clean_wheel_dataframe(data, label, path)
     # check if the time scale is in ms
-    if (data['re_ts'].iloc[-1] - data['re_ts'].iloc[0]) / 1e6 < 0:
+    if (data['re_ts'].iloc[-1] - data['re_ts'].iloc[0]) / 1e6 < 20:
         logger_.warning('Rotary encoder reset logs events in ms instead of us: ' +
                         'RE firmware needs upgrading and wheel velocity is potentially inaccurate')
         data['re_ts'] = data['re_ts'] * 1000
@@ -479,3 +472,50 @@ def get_port_events(events: dict, name: str = '') -> list:
     out = sorted(out)
 
     return out
+
+
+def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, max_shift=5,
+                       return_index=False):
+    """
+    Attempts to find matching timestamps in 2 time-series that have an offset, are drifting,
+    and are most likely incomplete: sizes don't have to match, some pulses may be missing
+    in any serie.
+    Only works with irregular time series as it relies on the derivative to match sync.
+    :param t0:
+    :param t1:
+    :param diff_threshold:
+    :param drift_threshold_ppm: (150)
+    :param max_shift: (200)
+    :param return_index (False)
+    :return:
+    """
+    nsync = min(t0.size, t1.size)
+    dt0 = np.diff(t0)
+    dt1 = np.diff(t1)
+    ind = np.zeros_like(dt0) * np.nan
+    i0 = 0
+    i1 = 0
+    cdt = np.nan  # the current time difference between the two series to compute drift
+    while i0 < (nsync - 1):
+        # look in the next max_shift events the ones whose derivative match
+        isearch = np.arange(i1, min(max_shift + i1, dt1.size))
+        dec = np.abs(dt0[i0] - dt1[isearch]) < diff_threshold
+        # another constraint is to check the dt for the maximum drift
+        if ~np.isnan(cdt):
+            drift_ppm = np.abs((cdt - (t0[i0] - t1[isearch])) / dt1[isearch]) * 1e6
+            dec = np.logical_and(dec, drift_ppm <= drift_threshold_ppm)
+        # if one is found
+        if np.any(dec):
+            ii1 = np.where(dec)[0][0]
+            ind[i0] = i1 + ii1
+            i1 += ii1 + 1
+            cdt = t0[i0 + 1] - t1[i1 + ii1]
+        i0 += 1
+    it0 = np.where(~np.isnan(ind))[0]
+    it1 = ind[it0].astype(np.int)
+    ind0 = np.unique(np.r_[it0, it0 + 1])
+    ind1 = np.unique(np.r_[it1, it1 + 1])
+    if return_index:
+        return t0[ind0], t1[ind1], ind0, ind1
+    else:
+        return t0[ind0], t1[ind1]
